import { useLexicalComposerContext } from "@lexical/react/LexicalComposerContext";
import { $createParagraphNode, $isParagraphNode, $isTextNode, SELECTION_CHANGE_COMMAND, TextNode } from "lexical";
import { useEffect } from "react";
import { $createHeaderMarkerNode, $createHeaderNode, $isHeaderMarkerNode, $isHeaderNode, HeaderMarkerNode, HeaderNode } from "../nodes/header_node";
import { mergeRegister } from "@lexical/utils";

const HEADER_REGEX = /^#+ /;

export function HeaderPlugin() {
    const [editor] = useLexicalComposerContext();

    useEffect(() => mergeRegister(
        // Create a header node if the text node matches the HEADER_REGEX
        editor.registerNodeTransform(TextNode, (textNode) => {
            const prevNode = textNode.getPreviousSibling();
            if (prevNode) return;
            const paragraphNode = textNode.getParent();
            if (!$isParagraphNode(paragraphNode)) return;

            const content = textNode.getTextContent();
            const regexMatch = content.match(HEADER_REGEX);
            if (!regexMatch) return;

            const children = paragraphNode.getChildren();

            const firstTextNode = children[0];
            if (!$isTextNode(firstTextNode)) return;

            const markerLength = regexMatch[0].length;
            const textNodes = firstTextNode.splitText(markerLength);

            const headerMarkerContent = textNodes[0];
            if (!headerMarkerContent) return;

            const headerNode = $createHeaderNode(markerLength - 1);
            const headerMarkerNode = $createHeaderMarkerNode();

            headerMarkerNode.append(headerMarkerContent);

            headerNode.append(headerMarkerNode);
            headerNode.append(...textNodes.slice(1));
            headerNode.append(...children.slice(1));

            paragraphNode.replace(headerNode, true);

            editor.dispatchCommand(SELECTION_CHANGE_COMMAND, undefined);
        }),
        // Remove headers if they don't match the HEADER_REGEX
        editor.registerNodeTransform(HeaderNode, (headerNode) => {
            const content = headerNode.getTextContent();
            if (content.match(HEADER_REGEX)) return;
            headerNode.replace($createParagraphNode(), true);
        }),
        // Remove header markers if they don't match the HEADER_REGEX
        editor.registerNodeTransform(HeaderMarkerNode, (node) => {
            const headerNode = node.getParent();
            const content = node.getTextContent();
            if ($isHeaderNode(headerNode) &&
                content.match(HEADER_REGEX) &&
                content.length - 1 == headerNode.getLevel()) {
                return;
            }

            node.getChildren().reverse().forEach(child => node.insertAfter(child));
            node.remove();
        }),
        // Remove header nodes without a header marker
        editor.registerNodeTransform(HeaderNode, (node) => {
            const children = node.getChildren();
            const headerMarker = children[0];
            if ($isHeaderMarkerNode(headerMarker)) return;

            node.replace($createParagraphNode(), true);
        }),
    ));

    return null;
}
